part of '/masamune.dart';

/// Create an extension method for [AppScopedValueOrAppRef] to handle the Query for the model.
///
/// モデル用のQueryを処理するための[AppScopedValueOrAppRef]の拡張メソッドを作成します。
extension MasamuneModelAppScopedValueOrAppRefExtensions
    on AppScopedValueOrAppRef {
  /// Retrieve the state-preserved [TModel] by passing the [ModelQueryBase] code generated by the builder.
  ///
  /// ビルダーによりコード生成された[ModelQueryBase]を渡すことにより状態を保持された[TModel]を取得します。
  ///
  /// ```dart
  /// final document = appRef.model(UserModel.document()); // Get the user document.
  /// final collection = appRef.model(UserModel.collection()); // Get the user collection.
  /// ```
  TModel model<TModel extends Listenable>(
    ModelQueryBase<TModel> query,
  ) {
    return this.query(query);
  }
}

/// Create an extension method for [QueryScopedValueRef] to handle the Query for the model.
///
/// モデル用のQueryを処理するための[QueryScopedValueRef]の拡張メソッドを作成します。
extension MasamuneModelQueryScopedValueRefExtensions
    on QueryScopedValueRef<AppScopedValueOrAppRef> {
  /// Retrieve the state-preserved [TModel] by passing the [ModelQueryBase] code generated by the builder.
  ///
  /// ビルダーによりコード生成された[ModelQueryBase]を渡すことにより状態を保持された[TModel]を取得します。
  ///
  /// ```dart
  /// final document = appRef.model(UserModel.document()); // Get the user document.
  /// final collection = appRef.model(UserModel.collection()); // Get the user collection.
  /// ```
  TModel model<TModel extends Listenable>(
    ModelQueryBase<TModel> query,
  ) {
    return this.query(query);
  }
}

/// Create extension methods for [RefHasApp] to handle Query for models.
///
/// モデル用のQueryを処理するための[RefHasApp]の拡張メソッドを作成します。
extension MasamuneModelRefHasAppExtensions on RefHasApp {
  @Deprecated(
    "It is no longer possible to use [model] by directly specifying [PageRef] or [WidgetRef]. Instead, use [ref.app.model] to specify the scope. [PageRef]や[WidgetRef]を直接指定しての[model]の利用はできなくなります。代わりに[ref.app.model]でスコープを指定しての利用を行ってください。",
  )
  TModel model<TModel extends Listenable>(
    ModelQueryBase<TModel> query,
  ) {
    return app.query(query);
  }
}

/// Base class for creating state-to-state usage queries for the model to be code-generated by the builder.
///
/// Basically, you can get classes that inherit from [DocumentBase] or [CollectionBase].
///
/// ビルダーによりコード生成するモデルの状態間利用クエリを作成するためのベースクラス。
///
/// 基本的には[DocumentBase]や[CollectionBase]を継承したクラスを取得することが出来ます。
abstract class ModelQueryBase<TModel extends Listenable>
    extends ChangeNotifierScopedQueryBase<TModel, AppScopedValueOrAppRef> {
  /// Base class for creating state-to-state usage queries for the model to be code-generated by the builder.
  ///
  /// Basically, you can get classes that inherit from [DocumentBase] or [CollectionBase].
  ///
  /// ビルダーによりコード生成するモデルの状態間利用クエリを作成するためのベースクラス。
  ///
  /// 基本的には[DocumentBase]や[CollectionBase]を継承したクラスを取得することが出来ます。
  const ModelQueryBase()
      : super(
          _provider,
        );

  static TModel _provider<TModel extends Listenable>(Ref ref) {
    throw UnimplementedError();
  }

  @override
  bool get autoDisposeWhenUnreferenced => false;
}
